@@ -12,10 +12,11 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
-import mock
+import datetime
 import testtools
 import yaml
 
+import mock
 from oslo_config import cfg
 
 from qinling import config
@@ -60,14 +61,6 @@ class TestKubernetesManager(base.DbTestCase):
         namespaces.items = [namespace]
         self.k8s_v1_api.list_namespace.return_value = namespaces
 
-        network_policy = mock.Mock()
-        network_policy.metadata.name = 'disable-interpods-connections'
-        network_policies = mock.Mock()
-        network_policies.items = [network_policy]
-        self.k8s_v1_ext.list_namespaced_network_policy.return_value = (
-            network_policies
-        )
-
         self.manager = k8s_manager.KubernetesManager(self.conf,
                                                      self.qinling_endpoint)
 
@@ -139,43 +132,6 @@ class TestKubernetesManager(base.DbTestCase):
         self.assertEqual(2, self.k8s_v1_api.list_namespace.call_count)
         self.k8s_v1_api.create_namespace.assert_not_called()
 
-    def test__ensure_network_policy(self):
-        # self.manager is not used in this test.
-        network_policies = mock.Mock()
-        network_policies.items = []
-        v1ext = self.k8s_v1_ext
-        v1ext.list_namespaced_network_policy.return_value = network_policies
-
-        k8s_manager.KubernetesManager(self.conf, self.qinling_endpoint)
-
-        network_policy_body = {
-            'apiVersion': 'extensions/v1beta1',
-            'kind': 'NetworkPolicy',
-            'metadata': {'name': 'disable-interpods-connections'},
-            'spec': {'pod_selector': {}}
-        }
-        v1ext.list_namespaced_network_policy.assert_called_with(
-            self.fake_namespace
-        )
-        v1ext.create_namespaced_network_policy.assert_called_once_with(
-            self.fake_namespace, network_policy_body)
-
-    def test__ensure_network_policy_not_create(self):
-        # self.manager is not used in this test.
-        item = mock.Mock()
-        item.metadata.name = 'disable-interpods-connections'
-        network_policies = mock.Mock()
-        network_policies.items = [item]
-        v1ext = self.k8s_v1_ext
-        v1ext.list_namespaced_network_policy.return_value = network_policies
-
-        k8s_manager.KubernetesManager(self.conf, self.qinling_endpoint)
-
-        v1ext.list_namespaced_network_policy.assert_called_with(
-            self.fake_namespace
-        )
-        v1ext.create_namespaced_network_policy.assert_not_called()
-
     def test_create_pool(self):
         ret = mock.Mock()
         ret.status.replicas = 5
@@ -202,7 +158,8 @@ class TestKubernetesManager(base.DbTestCase):
         )
         self.k8s_v1_ext.create_namespaced_deployment.assert_called_once_with(
             body=yaml.safe_load(deployment_body),
-            namespace=self.fake_namespace)
+            namespace=self.fake_namespace,
+            async_req=False)
         self.k8s_v1_ext.read_namespaced_deployment.assert_called_once_with(
             fake_deployment_name, self.fake_namespace)
 
@@ -263,8 +220,8 @@ class TestKubernetesManager(base.DbTestCase):
             self.fake_namespace,
             label_selector='runtime_id=%s' % fake_deployment_name)
         delete_service_calls = [
-            mock.call(svc1_name, self.fake_namespace, mock.ANY),
-            mock.call(svc2_name, self.fake_namespace, mock.ANY),
+            mock.call(svc1_name, self.fake_namespace),
+            mock.call(svc2_name, self.fake_namespace),
         ]
         self.k8s_v1_api.delete_namespaced_service.assert_has_calls(
             delete_service_calls)
@@ -499,6 +456,24 @@ class TestKubernetesManager(base.DbTestCase):
         self.k8s_v1_api.create_namespaced_pod.assert_called_once_with(
             self.fake_namespace, body=yaml.safe_load(pod_body))
 
+    def test_prepare_execution_with_image_pod_failed(self):
+        function_id = common.generate_unicode_uuid()
+        image = self.rand_name('image', prefix=self.prefix)
+        identifier = (
+            '%s-%s' % (common.generate_unicode_uuid(dashed=True), function_id)
+        )[:63]
+        self.k8s_v1_api.create_namespaced_pod.side_effect = RuntimeError
+
+        self.assertRaises(
+            exc.OrchestratorException,
+            self.manager.prepare_execution,
+            function_id,
+            0,
+            rlimit=self.rlimit,
+            image=image,
+            identifier=identifier,
+        )
+
     def test_prepare_execution_not_image_no_worker_available(self):
         ret_pods = mock.Mock()
         ret_pods.items = []
@@ -616,32 +591,110 @@ class TestKubernetesManager(base.DbTestCase):
 
     def test_run_execution_image_type_function(self):
         pod = mock.Mock()
+        status = mock.Mock()
+        status.state.terminated.finished_at = datetime.datetime(2018, 9, 4, 10,
+                                                                1, 50)
+        status.state.terminated.started_at = datetime.datetime(2018, 9, 4, 10,
+                                                               1, 40)
         pod.status.phase = 'Succeeded'
+        pod.status.container_statuses = [status]
         self.k8s_v1_api.read_namespaced_pod.return_value = pod
-        fake_output = 'fake output'
-        self.k8s_v1_api.read_namespaced_pod_log.return_value = fake_output
+        fake_log = 'fake log'
+        self.k8s_v1_api.read_namespaced_pod_log.return_value = fake_log
+        execution_id = common.generate_unicode_uuid()
+        function_id = common.generate_unicode_uuid()
+        identifier = 'fake_identifier'
+
+        result, output = self.manager.run_execution(execution_id, function_id,
+                                                    0, identifier=identifier)
+
+        self.k8s_v1_api.read_namespaced_pod.assert_called_once_with(
+            identifier, self.fake_namespace)
+        self.k8s_v1_api.read_namespaced_pod_log.assert_called_once_with(
+            identifier, self.fake_namespace)
+        self.assertTrue(result)
+
+        expected_output = {'duration': 10, 'logs': fake_log}
+        self.assertEqual(expected_output, output)
+
+    def test_run_execution_image_type_function_retry(self):
+        pod1 = mock.Mock()
+        pod1.status.phase = ''
+        pod2 = mock.Mock()
+        status = mock.Mock()
+        status.state.terminated.finished_at = datetime.datetime(2018, 9, 4, 10,
+                                                                1, 50)
+        status.state.terminated.started_at = datetime.datetime(2018, 9, 4, 10,
+                                                               1, 40)
+        pod2.status.phase = 'Succeeded'
+        pod2.status.container_statuses = [status]
+        self.k8s_v1_api.read_namespaced_pod.side_effect = [pod1, pod2]
+        fake_log = 'fake log'
+        self.k8s_v1_api.read_namespaced_pod_log.return_value = fake_log
         execution_id = common.generate_unicode_uuid()
         function_id = common.generate_unicode_uuid()
 
         result, output = self.manager.run_execution(execution_id, function_id,
-                                                    0)
+                                                    0, timeout=5)
 
-        self.k8s_v1_api.read_namespaced_pod.assert_called_once_with(
-            None, self.fake_namespace)
+        self.assertEqual(2, self.k8s_v1_api.read_namespaced_pod.call_count)
         self.k8s_v1_api.read_namespaced_pod_log.assert_called_once_with(
             None, self.fake_namespace)
         self.assertTrue(result)
-        self.assertEqual(fake_output, output)
+
+        expected_output = {'duration': 10, 'logs': fake_log}
+        self.assertEqual(expected_output, output)
+
+    def test_run_execution_image_type_function_timeout(self):
+        execution_id = common.generate_unicode_uuid()
+        function_id = common.generate_unicode_uuid()
+        pod1 = mock.Mock()
+        pod1.status.phase = ''
+        self.k8s_v1_api.read_namespaced_pod.return_value = pod1
+
+        result, output = self.manager.run_execution(
+            execution_id, function_id, 0,
+            identifier='fake_identifier',
+            timeout=1
+        )
+
+        self.assertFalse(result)
+
+        expected_output = {
+            'output': 'Function execution timeout.',
+            'duration': 1
+        }
+        self.assertEqual(expected_output, output)
+
+    def test_run_execution_image_type_function_read_pod_exception(self):
+        self.k8s_v1_api.read_namespaced_pod.side_effect = RuntimeError
+        execution_id = common.generate_unicode_uuid()
+        function_id = common.generate_unicode_uuid()
+
+        result, output = self.manager.run_execution(execution_id, function_id,
+                                                    0, timeout=5)
+
+        self.k8s_v1_api.read_namespaced_pod.assert_called_once_with(
+            None, self.fake_namespace)
+        self.k8s_v1_api.read_namespaced_pod_log.assert_not_called()
+        self.assertFalse(result)
+
+        expected_output = {
+            'output': 'Function execution failed.',
+            'duration': 0
+        }
+        self.assertEqual(expected_output, output)
 
     @mock.patch('qinling.engine.utils.url_request')
     def test_run_execution_version_0(self, mock_request):
         mock_request.return_value = (True, 'fake output')
         execution_id = common.generate_unicode_uuid()
         function_id = common.generate_unicode_uuid()
+        timeout = 3
 
         result, output = self.manager.run_execution(
             execution_id, function_id, 0, rlimit=self.rlimit,
-            service_url='FAKE_URL'
+            service_url='FAKE_URL', timeout=timeout
         )
 
         download_url = ('http://127.0.0.1:7070/v1/functions/%s?download=true'
@@ -656,46 +709,13 @@ class TestKubernetesManager(base.DbTestCase):
             'entry': 'main.main',
             'download_url': download_url,
             'request_id': self.ctx.request_id,
+            'timeout': timeout,
         }
 
         mock_request.assert_called_once_with(
             self.manager.session, 'FAKE_URL/execute', body=data
         )
 
-    def test_run_execution_no_service_url_retry(self):
-        pod1 = mock.Mock()
-        pod1.status.phase = ''
-        pod2 = mock.Mock()
-        pod2.status.phase = 'Succeeded'
-        self.k8s_v1_api.read_namespaced_pod.side_effect = [pod1, pod2]
-        fake_output = 'fake output'
-        self.k8s_v1_api.read_namespaced_pod_log.return_value = fake_output
-        execution_id = common.generate_unicode_uuid()
-        function_id = common.generate_unicode_uuid()
-
-        result, output = self.manager.run_execution(execution_id, function_id,
-                                                    0)
-
-        self.assertEqual(2, self.k8s_v1_api.read_namespaced_pod.call_count)
-        self.k8s_v1_api.read_namespaced_pod_log.assert_called_once_with(
-            None, self.fake_namespace)
-        self.assertTrue(result)
-        self.assertEqual(fake_output, output)
-
-    def test_run_execution_no_service_url_read_pod_exception(self):
-        self.k8s_v1_api.read_namespaced_pod.side_effect = RuntimeError
-        execution_id = common.generate_unicode_uuid()
-        function_id = common.generate_unicode_uuid()
-
-        result, output = self.manager.run_execution(execution_id, function_id,
-                                                    0)
-
-        self.k8s_v1_api.read_namespaced_pod.assert_called_once_with(
-            None, self.fake_namespace)
-        self.k8s_v1_api.read_namespaced_pod_log.assert_not_called()
-        self.assertFalse(result)
-        self.assertEqual({'error': 'Function execution failed.'}, output)
-
     def test_delete_function(self):
         # Deleting namespaced service is also tested in this.
         svc1 = mock.Mock()
@@ -723,8 +743,8 @@ class TestKubernetesManager(base.DbTestCase):
         )
 
         delete_service_calls = [
-            mock.call(svc1_name, self.fake_namespace, mock.ANY),
-            mock.call(svc2_name, self.fake_namespace, mock.ANY)
+            mock.call(svc1_name, self.fake_namespace),
+            mock.call(svc2_name, self.fake_namespace)
         ]
         self.k8s_v1_api.delete_namespaced_service.assert_has_calls(
             delete_service_calls)
@@ -901,4 +921,5 @@ class TestKubernetesManager(base.DbTestCase):
         self.manager.delete_worker(pod_name)
 
         self.k8s_v1_api.delete_namespaced_pod.assert_called_once_with(
-            pod_name, self.fake_namespace, {})
+            pod_name, self.fake_namespace
+        )
